import datetime
import json
from typing import Any
from uuid import UUID

from pydantic_core import InitErrorDetails

from canvas_generated.messages.effects_pb2 import EffectType
from canvas_sdk.effects import Effect
from canvas_sdk.effects.note.base import NoteOrAppointmentABC
from canvas_sdk.v1.data import Note as NoteModel
from canvas_sdk.v1.data import NoteType, Patient
from canvas_sdk.v1.data.note import NoteStates, NoteTypeCategories

TRANSITION_STATE_MATRIX = {
    NoteStates.NEW: [NoteStates.LOCKED, NoteStates.PUSHED],
    NoteStates.LOCKED: [NoteStates.SIGNED, NoteStates.UNLOCKED],
    NoteStates.UNLOCKED: [NoteStates.LOCKED, NoteStates.PUSHED],
    NoteStates.SIGNED: [NoteStates.UNLOCKED, NoteStates.SIGNED],
    NoteStates.PUSHED: [NoteStates.LOCKED, NoteStates.PUSHED],
}

ACTION_STATE_MATRIX = {
    "lock": NoteStates.LOCKED,
    "unlock": NoteStates.UNLOCKED,
    "sign": NoteStates.SIGNED,
    "push_charges": NoteStates.PUSHED,
    "check_in": NoteStates.CONVERTED,
    "no_show": NoteStates.NOSHOW,
}


class Note(NoteOrAppointmentABC):
    """
    Effect to create a visit note.

    Attributes:
        note_type_id (UUID | str): The ID of the note type.
        datetime_of_service (datetime.datetime): The date and time of the service.
        patient_id (str): The ID of the patient.
    """

    class Meta:
        effect_type = "NOTE"

    note_type_id: UUID | str | None = None
    datetime_of_service: datetime.datetime | None = None
    patient_id: str | None = None
    title: str | None = None

    def push_charges(self) -> Effect:
        """Pushes BillingLineItems from the Note to the associated Claim. Identicial to clicking the Push Charges button in the note footer."""
        self._validate_before_effect("push_charges")
        return Effect(
            type=EffectType.PUSH_NOTE_CHARGES,
            payload=json.dumps({"data": {"note": str(self.instance_id)}}),
        )

    def unlock(self) -> Effect:
        """Unlocks the note to allow further edits."""
        self._validate_before_effect("unlock")
        return Effect(
            type=EffectType.UNLOCK_NOTE,
            payload=json.dumps({"data": {"note": str(self.instance_id)}}),
        )

    def lock(self) -> Effect:
        """Locks the note to prevent further edits."""
        self._validate_before_effect("lock")
        return Effect(
            type=EffectType.LOCK_NOTE,
            payload=json.dumps({"data": {"note": str(self.instance_id)}}),
        )

    def sign(self) -> Effect:
        """Signs the note."""
        self._validate_before_effect("sign")
        return Effect(
            type=EffectType.SIGN_NOTE,
            payload=json.dumps({"data": {"note": str(self.instance_id)}}),
        )

    def check_in(self) -> Effect:
        """Mark the note as checked-in."""
        self._validate_before_effect("check_in")
        return Effect(
            type=EffectType.CHECK_IN_NOTE,
            payload=json.dumps({"data": {"note": str(self.instance_id)}}),
        )

    def no_show(self) -> Effect:
        """Mark the note as no-show."""
        self._validate_before_effect("no_show")
        return Effect(
            type=EffectType.NO_SHOW_NOTE,
            payload=json.dumps({"data": {"note": str(self.instance_id)}}),
        )

    def _validate_state_transition(
        self, note: NoteModel, next_state: NoteStates
    ) -> tuple[bool, InitErrorDetails | None]:
        """Validates state transitions for the note."""
        current_state = note.current_state.state if note.current_state else None

        if not current_state:
            return False, self._create_error_detail(
                "value", "Unsupported state transitions", next_state
            )

        is_sig_required = note.note_type_version.is_sig_required
        if next_state == NoteStates.SIGNED and not is_sig_required:
            return False, self._create_error_detail(
                "value", "Cannot sign a note that does not require a signature.", next_state
            )

        if (
            next_state == NoteStates.CONVERTED or next_state == NoteStates.NOSHOW
        ) and note.note_type_version.category != NoteTypeCategories.APPOINTMENT:
            return False, self._create_error_detail(
                "value",
                "Only appointments can be checked in or marked as no-show.",
                next_state,
            )

        return True, None

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        """
        Validates the note type category and returns a list of error details if validation fails.

        Args:
            method (Any): The method being validated.

        Returns:
            list[InitErrorDetails]: A list of error details for invalid note type categories.
        """
        errors = super()._get_error_details(method)

        if method in ACTION_STATE_MATRIX:
            if not self.instance_id:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        "Field 'instance_id' is required.",
                        None,
                    )
                )
                return errors
            elif not (note := NoteModel.objects.filter(id=self.instance_id).first()):
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note with ID {self.instance_id} does not exist.",
                        self.instance_id,
                    )
                )
                return errors

            if note.note_type_version.category in (
                NoteTypeCategories.LETTER,
                NoteTypeCategories.MESSAGE,
            ):
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note with note type {note.note_type_version.name} cannot perform action '{method}'.",
                        note.note_type_version,
                    )
                )
                return errors

            if method == "push_charges" and (
                not note.note_type_version or not note.note_type_version.is_billable
            ):
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note with note type {note.note_type_version} is not billable and has no associated claim.",
                        note.note_type_version,
                    )
                )
                return errors

            if method in ACTION_STATE_MATRIX:
                _, error = self._validate_state_transition(note, ACTION_STATE_MATRIX[method])

                if error:
                    errors.append(error)

            return errors

        if method == "create":
            if not self.note_type_id:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        "Field 'note_type_id' is required to create a note.",
                        None,
                    )
                )

            if not self.datetime_of_service:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        "Field 'datetime_of_service' is required to create a note.",
                        None,
                    )
                )

            if not self.patient_id:
                errors.append(
                    self._create_error_detail(
                        "missing",
                        "Field 'patient_id' is required to create a note.",
                        None,
                    )
                )

            if self.instance_id and NoteModel.objects.filter(id=self.instance_id).exists():
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note with ID {self.instance_id} already exists.",
                        self.instance_id,
                    )
                )

        elif method == "update":
            if self.note_type_id:
                errors.append(
                    self._create_error_detail(
                        "invalid",
                        "Field 'note_type_id' cannot be updated for a note.",
                        self.note_type_id,
                    )
                )
            if self.patient_id:
                errors.append(
                    self._create_error_detail(
                        "invalid",
                        "Field 'patient_id' cannot be updated for a note.",
                        self.patient_id,
                    )
                )

            if self.instance_id and not NoteModel.objects.filter(id=self.instance_id).exists():
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note with ID {self.instance_id} does not exist.",
                        self.instance_id,
                    )
                )

        if self.note_type_id:
            note_type_category = (
                NoteType.objects.values_list("category", flat=True)
                .filter(id=self.note_type_id)
                .first()
            )

            if not note_type_category:
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Note type with ID {self.note_type_id} does not exist.",
                        self.note_type_id,
                    )
                )
            elif note_type_category in (
                NoteTypeCategories.APPOINTMENT,
                NoteTypeCategories.SCHEDULE_EVENT,
                NoteTypeCategories.MESSAGE,
                NoteTypeCategories.LETTER,
            ):
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Visit note type cannot be of type: {note_type_category}.",
                        self.note_type_id,
                    )
                )

        if self.patient_id and not Patient.objects.filter(id=self.patient_id).exists():
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Patient with ID {self.patient_id} does not exist.",
                    self.patient_id,
                )
            )

        return errors


__exports__ = ("Note",)



from typing import Any

from pydantic import Field

from canvas_sdk.effects import EffectType, _BaseEffect


class BatchOriginateCommandEffect(_BaseEffect):
    """An Effect that will originate multiple commands in a batch operation."""

    class Meta:
        effect_type = EffectType.BATCH_ORIGINATE_COMMANDS

    commands: list = Field(min_length=1)

    @property
    def values(self) -> dict[str, Any]:
        """The BatchOriginateCommandEffect's values."""
        return {"commands": [command._origination_payload_for_batch() for command in self.commands]}


__exports__ = ("BatchOriginateCommandEffect",)



from typing import cast

from django.db import models

from canvas_sdk.v1.data.base import (
    BaseModelManager,
    BaseQuerySet,
    CommittableQuerySetMixin,
    ForPatientQuerySetMixin,
    IdentifiableModel,
    TimestampedModel,
)


class IntervalUnit(models.TextChoices):
    """ProtocolOverride cycle IntervalUnit."""

    DAYS = "days", "days"
    MONTHS = "months", "months"
    YEARS = "years", "years"


class Status(models.TextChoices):
    """ProtocolOverride Status."""

    ACTIVE = "active", "active"
    INACTIVE = "inactive", "inactive"


class ProtocolOverrideQuerySet(ForPatientQuerySetMixin, CommittableQuerySetMixin, BaseQuerySet):
    """ProtocolOverrideQuerySet."""

    pass


ProtocolOverrideManager = BaseModelManager.from_queryset(ProtocolOverrideQuerySet)


class ProtocolOverride(TimestampedModel, IdentifiableModel):
    """ProtocolOverride."""

    class Meta:
        db_table = "canvas_sdk_data_api_protocoloverride_001"

    objects = cast(ProtocolOverrideQuerySet, ProtocolOverrideManager())

    deleted = models.BooleanField()
    committer = models.ForeignKey(
        "v1.CanvasUser", on_delete=models.DO_NOTHING, null=True, related_name="+"
    )
    entered_in_error = models.ForeignKey(
        "v1.CanvasUser", on_delete=models.DO_NOTHING, null=True, related_name="+"
    )
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, related_name="protocol_overrides", null=True
    )
    protocol_key = models.CharField(max_length=250)
    is_adjustment = models.BooleanField()
    reference_date = models.DateTimeField()
    cycle_in_days = models.IntegerField()
    is_snooze = models.BooleanField()
    snooze_date = models.DateField()
    snoozed_days = models.IntegerField()
    # reason_id = models.BigIntegerField()
    snooze_comment = models.TextField()
    narrative = models.CharField(max_length=512)
    # note_id = models.BigIntegerField()
    cycle_quantity = models.IntegerField()
    cycle_unit = models.CharField(choices=IntervalUnit.choices, max_length=20)
    status = models.CharField(choices=Status.choices, max_length=20)


__exports__ = (
    "IntervalUnit",
    "Status",
    "ProtocolOverride",
)



import json
from enum import StrEnum
from typing import Any
from uuid import UUID

from canvas_generated.messages.effects_pb2 import Effect
from canvas_sdk.effects import EffectType, _BaseEffect


class CalendarType(StrEnum):
    """Calendar type."""

    Clinic = "Clinic"
    Administrative = "Admin"


class Calendar(_BaseEffect):
    """Effect to create a Calendar."""

    id: str | UUID | None = None
    provider: str | UUID
    type: CalendarType
    location: str | UUID | None = None
    description: str | None = None

    @property
    def values(self) -> dict[str, Any]:
        """The Calendar's values."""
        return {
            "id": self.id,
            "provider": self.provider,
            "type": self.type,
            "location": self.location,
            "description": self.description,
        }

    def create(self) -> Effect:
        """Send a CREATE effect for the calendar."""
        self._validate_before_effect("create")

        return Effect(
            type=EffectType.CALENDAR__CREATE,
            payload=json.dumps(
                {
                    "data": self.values,
                }
            ),
        )


__exports__ = (
    "Calendar",
    "CalendarType",
)



from __future__ import annotations

from typing import NamedTuple


class LlmTurn(NamedTuple):
    """A single conversation turn in an LLM interaction.

    Attributes:
        role: The role of the speaker (e.g., 'system', 'user', 'model').
        text: List of text strings for this turn.
    """

    role: str
    text: list[str]

    def to_dict(self) -> dict:
        """Convert the turn to a dictionary representation.

        Returns:
            Dictionary with 'role' and 'text' keys.
        """
        return {
            "role": self.role,
            "text": self.text,
        }

    @classmethod
    def load_from_dict(cls, dict_list: list[dict]) -> list[LlmTurn]:
        """Load a list of turns from a list of dictionaries.

        Args:
            dict_list: List of dictionaries, each containing 'role' and 'text' keys.

        Returns:
            List of LlmTurn instances created from the dictionaries.
        """
        return [
            LlmTurn(
                role=json_object.get("role") or "",
                text=json_object.get("text") or [],
            )
            for json_object in dict_list
        ]


__exports__ = ("LlmTurn",)



from ..value_set import ValueSet


class TobaccoUseScreening(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments to screen for tobacco use.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment for tobacco use, including smoking and smoke-less tobacco products.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS138v10, CMS249v4
    """

    VALUE_SET_NAME = "Tobacco Use Screening"
    OID = "2.16.840.1.113883.3.526.3.1278"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "39240-7",  # Tobacco use status CPHS
        "68535-4",  # Have you used tobacco in the last 30 days [SAMHSA]
        "68536-2",  # Have you used smokeless tobacco product in the last 30 days [SAMHSA]
        "72166-2",  # Tobacco smoking status
    }


class FallsScreening(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments using a falls screening tool.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment for fall risk.

    **Exclusion Criteria:** Excludes concepts that represent an assessment for falls for children.

    ** Used in:** CMS139v10
    """

    VALUE_SET_NAME = "Falls Screening"
    OID = "2.16.840.1.113883.3.464.1003.118.12.1028"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "52552-7",  # Falls in the past year [CMS Assessment]
        "57254-5",  # Standardized fall risk assessment was conducted [CMS Assessment]
        "59454-9",  # History of falling; immediate or within 3 months [Morse Fall Scale]
        "73830-2",  # Fall risk assessment
    }


class StandardizedToolsForAssessmentOfCognition(ValueSet):
    """
        **Clinical Focus:** The purpose of this value set is to define concepts for assessments representing total score results for standardized tools used for the evaluation of cognition.

        **Data Element Scope:** This value set may use a model element related to Assessment.

        **Inclusion Criteria:** Includes concepts that describe assessments with total score results for the following standardized tools:
    -Blessed Orientation-Memory-Concentration Test (BOMC)
    -Montreal Cognitive Assessment (MoCA)
    -St. Louis University Mental Status Examination (SLUMS)
    -Mini-Mental State Examination (MMSE) [Note: The MMSE has not been well validated for non-Alzheimer's dementias]
    -Short Informant Questionnaire on Cognitive Decline in the Elderly (IQCODE)
    -Ascertain Dementia 8 (AD8) Questionnaire
    -Minimum Data Set (MDS) Brief Interview of Mental Status (BIMS) [Note: Validated for use with nursing home patients only]
    -Formal neuropsychological evaluation
    -Mini-Cog.

        **Exclusion Criteria:** No exclusions.

        ** Used in:** CMS149v10
    """

    VALUE_SET_NAME = "Standardized Tools for Assessment of Cognition"
    OID = "2.16.840.1.113883.3.526.3.1006"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "58151-2",  # Prior assessment Brief Interview for Mental Status (BIMS) summary score [MDSv3]
        "71492-3",  # Total score [SLUMS]
        "71493-1",  # Total score [IQCODE]
        "71722-3",  # Total score [AD8]
        "72106-8",  # Total score [MMSE]
        "72172-0",  # Total score [MoCA]
        "72173-8",  # Total score [BOMC]
        "72233-0",  # Total score [Mini-Cog]
    }


class SexuallyActive(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments to indicate vaginal intercourse.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment for intercourse.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS153v10
    """

    VALUE_SET_NAME = "Sexually Active"
    OID = "2.16.840.1.113883.3.464.1003.121.12.1040"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "64728-9",  # Have you ever had vaginal intercourse [PhenX]
    }


class StandardizedPainAssessmentTool(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments using pain-focused tools or instruments to quantify pain intensity.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment of pain intensity.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS157v10
    """

    VALUE_SET_NAME = "Standardized Pain Assessment Tool"
    OID = "2.16.840.1.113883.3.526.3.1028"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "38208-5",  # Pain severity - Reported
        "38214-3",  # Pain severity [Score] Visual analog score
        "38221-8",  # Pain severity Wong-Baker FACES pain rating scale
        "72514-3",  # Pain severity - 0-10 verbal numeric rating [Score] - Reported
        "77565-0",  # Pain interference score [BPI Short Form]
    }


class Phq9AndPhq9MTools(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for the assessments of PHQ 9 and PHQ 9M resulting in a completed depression assessment scores for adults and adolescents.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent a completed assessment using PHQ 9 and PHQ 9M depression assessment tools for adults and adolescents with a summary score.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS159v10
    """

    VALUE_SET_NAME = "PHQ 9 and PHQ 9M Tools"
    OID = "2.16.840.1.113883.3.67.1.101.1.263"
    DEFINITION_VERSION = "20210219"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "44261-6",  # Patient Health Questionnaire 9 item (PHQ-9) total score [Reported]
        "89204-2",  # Patient Health Questionnaire-9: Modified for Teens total score [Reported.PHQ.Teen]
    }


class AverageNumberOfDrinksPerDrinkingDay(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments measuring the number of alcoholic drinks per drinking day.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment of a quantitative observation of reported number of alcoholic drinks per drinking day.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS249v4
    """

    VALUE_SET_NAME = "Average Number of Drinks per Drinking Day"
    OID = "2.16.840.1.113883.3.464.1003.106.12.1018"
    DEFINITION_VERSION = "20180310"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    LOINC = {
        "11287-0",  # Alcoholic drinks per drinking day - Reported
    }


class HistoryOfHipFractureInParent(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for H145 assessments of a family history of hip fracture in a parent.

    **Data Element Scope:** This value set may use a model element related to Assessment.

    **Inclusion Criteria:** Includes concepts that represent an assessment for family history of hip fracture in a parent.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS249v4
    """

    VALUE_SET_NAME = "History of hip fracture in parent"
    OID = "2.16.840.1.113883.3.464.1003.113.12.1040"
    DEFINITION_VERSION = "20180310"
    EXPANSION_VERSION = "eCQM Update 2021-05-06"

    SNOMEDCT = {
        "391096007",  # Family history: Maternal hip fracture (situation)
        "416072008",  # Family history: maternal hip fracture before age 75 (situation)
        "445121000124105",  # Family history of paternal hip fracture (situation)
        "445501000124107",  # Family history of hip fracture in parent (situation)
    }


__exports__ = (
    "AverageNumberOfDrinksPerDrinkingDay",
    "FallsScreening",
    "HistoryOfHipFractureInParent",
    "Phq9AndPhq9MTools",
    "SexuallyActive",
    "StandardizedPainAssessmentTool",
    "StandardizedToolsForAssessmentOfCognition",
    "TobaccoUseScreening",
)



from canvas_sdk.commands.base import _BaseCommand


class PlanCommand(_BaseCommand):
    """A class for managing a Plan command within a specific note."""

    class Meta:
        key = "plan"

    narrative: str = ""


__exports__ = ("PlanCommand",)



from canvas_sdk.clients.llms.structures.llm_response import LlmResponse
from canvas_sdk.clients.llms.structures.llm_tokens import LlmTokens
from canvas_sdk.clients.llms.structures.llm_turn import LlmTurn

__all__ = __exports__ = (
    "LlmResponse",
    "LlmTokens",
    "LlmTurn",
)



import datetime
import json

import factory
from django.utils import timezone
from factory.fuzzy import FuzzyDate

from canvas_sdk.v1.data import ImagingOrder, ImagingReport, ImagingReview
from canvas_sdk.v1.data.common import (
    DocumentReviewMode,
    OrderStatus,
    ReviewPatientCommunicationMethod,
    ReviewStatus,
)
from canvas_sdk.v1.data.imaging import ImagingReport as ImagingReportModel


class ImagingOrderFactory(factory.django.DjangoModelFactory[ImagingOrder]):
    """Factory for creating an ImagingOrder."""

    class Meta:
        model = ImagingOrder

    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    note = factory.SubFactory("canvas_sdk.test_utils.factories.NoteFactory")
    imaging = factory.Faker("text", max_nb_chars=1024)
    imaging_center = None  # Optional field
    note_to_radiologist = factory.Faker("text", max_nb_chars=1024)
    internal_comment = factory.Faker("text", max_nb_chars=1024)
    status = OrderStatus.REQUESTED
    date_time_ordered = factory.LazyFunction(lambda: timezone.now())
    priority = factory.Faker("random_element", elements=["routine", "urgent", "stat"])
    ordering_provider = factory.SubFactory("canvas_sdk.test_utils.factories.StaffFactory")
    delegated = False
    task_ids = factory.LazyFunction(lambda: json.dumps([]))


class ImagingReviewFactory(factory.django.DjangoModelFactory[ImagingReview]):
    """Factory for creating an ImagingReview."""

    class Meta:
        model = ImagingReview

    patient_communication_method = ReviewPatientCommunicationMethod.DELEGATED_CALL_CAN_LEAVE_MESSAGE
    internal_comment = factory.Faker("text", max_nb_chars=2048)
    message_to_patient = factory.Faker("text", max_nb_chars=2048)
    is_released_to_patient = False
    status = ReviewStatus.STATUS_REVIEWING
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")


class ImagingReportFactory(factory.django.DjangoModelFactory[ImagingReport]):
    """Factory for creating an ImagingReport."""

    class Meta:
        model = ImagingReport

    review_mode = DocumentReviewMode.REVIEW_REQUIRED
    junked = False
    requires_signature = False
    assigned_date = factory.LazyFunction(lambda: timezone.now())
    patient = factory.SubFactory("canvas_sdk.test_utils.factories.PatientFactory")
    order = None  # Optional field
    source = ImagingReportModel.ImagingReportSource.DIRECTLY_REPORT
    name = factory.Faker("text", max_nb_chars=255)
    result_date = FuzzyDate(
        start_date=datetime.date.today() - datetime.timedelta(days=365),
        end_date=datetime.date.today(),
    )
    original_date = FuzzyDate(
        start_date=datetime.date.today() - datetime.timedelta(days=365),
        end_date=datetime.date.today(),
    )
    review = None  # Optional field



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: canvas_generated/services/plugin_runner.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from canvas_generated.messages import events_pb2 as canvas__generated_dot_messages_dot_events__pb2
from canvas_generated.messages import plugins_pb2 as canvas__generated_dot_messages_dot_plugins__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-canvas_generated/services/plugin_runner.proto\x12\x06\x63\x61nvas\x1a&canvas_generated/messages/events.proto\x1a\'canvas_generated/messages/plugins.proto2\x85\x02\n\x0cPluginRunner\x12\x35\n\x0bHandleEvent\x12\r.canvas.Event\x1a\x15.canvas.EventResponse0\x01\x12@\n\rReloadPlugins\x12\x15.ReloadPluginsRequest\x1a\x16.ReloadPluginsResponse0\x01\x12=\n\x0cReloadPlugin\x12\x14.ReloadPluginRequest\x1a\x15.ReloadPluginResponse0\x01\x12=\n\x0cUnloadPlugin\x12\x14.UnloadPluginRequest\x1a\x15.UnloadPluginResponse0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'canvas_generated.services.plugin_runner_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_PLUGINRUNNER']._serialized_start=139
  _globals['_PLUGINRUNNER']._serialized_end=400
# @@protoc_insertion_point(module_scope)
